let EventEmitter = require('events'),
		Util = require('util');

function _getMongo(options, fn) {
  let MongoClient = require('mongodb').MongoClient,
      user = encodeURIComponent(options.user),
      password = encodeURIComponent(options.password),
      authMechanism = encodeURIComponent(options.authMechanism),
      authSource = encodeURIComponent(options.authSource),
      database = encodeURIComponent(options.database),
      url = `${options.schema}${user}:${password}@${options.host}:${options.port}/${database}?authMechanism=${authMechanism}&authSource=${authSource}`;
  // Use connect method to connect to the Server
  MongoClient.connect(url, function(err, db) {
    if(err) {
      console.log(`xceling-database.js: ${err.message}`)
      return fn && fn(`Mongo authentication error: ${err.message}`, null)
    }
    return fn && fn(null, db)
  });
}

function _getMySQL(options) {
	let mysql = require('mysql').createPool({
		connectionLimit   : options.connection_limit,
	  database          : options.database,
	  host              : options.host,
	  password          : options.password,
	  supportBigNumbers : options.supportBigNumbers,
	  user              : options.user
	});
	return mysql;
}

module.exports = class Database {

	constructor(options) {
		let _this = this;
		if(!options || typeof options !== 'object') {
			console.log('xceling-database error: a JSON object containing configuration information is required: see database information in site configuration file.');
			return null;
		}
		for(let db of options) {
			if(!_this[db.type]) {
				_this[db.type] = [];
			}
			if(db.type.toLowerCase() == 'mysql') {
				_mysql[_mysql.length] = _getMySQL(db);
			} else if(db.type.toLowerCase() == 'mongo') {
				_getMongo(db, function(err, db) {
					if(!err) {
						_this.mongo[_this.mongo.length] = db;
					}
				});
			}
		}
	}
	// Instance getters and setters
	get Mongo() {
		try {
			return this.mongo[0];
		} catch(e) {
			return null;
		}
	}
	get MySQL() {
		try {
			return this.mysql[0];
		} catch(e) {
			return null;
		}
	}
	// End instance getters and setters.
	// Instance methods

	/**
	 * Retrieves a database object specified by the type and name.
	 *
	 * @param {Object} args, required, must contain the following elements: {type : [database type], name : [database name]}.
	 * @api public
	 */
	getDatabaseByName(options) {
		if(!options) {
			return null;
		}
		if(options.type == 'mongo') {
			for(let i = 0; i < _mongo.length; i++) {
				if(_mongo[i].databaseName == options.name) {
					return _mongo[i];
				}
			}
			return null;
		} else if(options.type == 'mysql') {
			for(let i = 0; i < _mysql.length; i++) {
				if(_mysql[i].name == options.name) {
					return _mysql[i];
				}
			}
			return null;
		} else {
			return null;
		}
	}
	// End instance methods

  // Required for module hot swapping
	static get ModulePath() {
	  return false; //__filename;
	}
		
}
